from .base import FileInput, FileInputTemplate
from .collection import FileInputCollectionTemplate, FileInputCollection
from .copy import CopyFileInput, CopyFileInputTemplate
from .general import FileInputType, load_input_template
from .link import LinkFileInput, LinkFileInputTemplate
from .tag import TagFileInput, TagFileInputTemplate
