from .base import FileOutputTemplate, FileOutput
from .collection import FileOutputCollectionTemplate, FileOutputCollection
from .copy import CopyFileOutputTemplate, CopyFileOutput
from .general import FileOutputType, load_output_template
from .tag import TagFileOutputTemplate, TagFileOutput
